@@ -1,4 +1,4 @@
-/* $EPIC: ignore.c,v 1.15 2005/04/03 15:36:08 jnelson Exp $ */
+/* $EPIC: ignore.c,v 1.37 2013/03/27 23:12:06 jnelson Exp $ */
 /*
  * ignore.c: handles the ingore command for irc 
  *
@@ -52,10 +52,6 @@
  *   2) Exceptive ignore (IGNORE_DONT) - A message from this person shall be 
  *      considered important enough to be exempt from all other ignore rules.
  *      Ignores of this type are "exceptions" to ignores of other types.
- *   3) Highlight ignore (IGNORE_HIGH) - A message from this person shall not
- *      only be exempt from all other ignore rules, but EPIC's default output
- *      shall be embellished to indicate to the user that a person of import
- *      has sent a message.
  *
  * Each Ignore type has (currently) many "levels" through which all messages
  * are sorted:
@@ -65,13 +61,12 @@
  *	WALLOPS		WALLOPs messages sent to anybody.
  *	INVITES		INVITEs sent to you
  *	NOTICES		NOTICEs sent to anybody
- *	NOTES		"NOTE" notices sent to you.
  *	CTCPS		CTCP requests or CTCP replies sent to anybody
  *	TOPICS		TOPIC changes by anybody
  *	NICKS		NICK changes by anybody
  *	JOINS		JOINs to any channel by anybody
  *	PARTS		PARTs from any channel by anybody
- *	CRAP		QUITs, MODEs, KICKs and PONGs.
+ *	OTHER		QUITs, MODEs, KICKs and PONGs.
  *
  * In addition, the user can use these shorthand level descriptions:
  *	ALL		All of the above.
@@ -103,11 +98,11 @@
 #include "timer.h"
 #include "functions.h"
 #include "window.h"
+#include "reg.h"
 
 #define IGNORE_REMOVE 	-1
 #define IGNORE_SUPPRESS  0
 #define IGNORE_DONT 	 1
-#define IGNORE_HIGH 	 2
 
 
 /*
@@ -119,24 +114,24 @@ typedef struct	IgnoreStru
 	struct	IgnoreStru *next;
 	char	*nick;			/* What is being ignored */
 	int	refnum;			/* The refnum for internal use */
-	int	type;			/* Suppressive ignores */
-	int	dont;			/* Exceptional ignores */
-	int	high;			/* Highlight ignores */
+	Mask	type;			/* Suppressive ignores */
+	Mask	dont;			/* Exceptional ignores */
 	int	counter;		/* How many times it has triggered */
 	Timeval	creation;		/* When it was created */
 	Timeval	last_used;		/* When it was last ``triggered'' */
 	Timeval	expiration;		/* When this ignore expires */
 	char	*reason;
+	int	enabled;
 }	Ignore;
 
 /* ignored_nicks: pointer to the head of the ignore list */
 static	Ignore *ignored_nicks = NULL;
 static	int	global_ignore_refnum = 0;
+static	int	ignores_are_suspended = 0;
 
 static void	expire_ignores			(void);
 static const char *	get_ignore_types 		(Ignore *item, int);
-static int	change_ignore_mask_by_desc 	(const char *, int *, int *, 
-						 int *, char **, Timeval *);
+static int	change_ignore_mask_by_desc (const char *type, Mask *do_mask, Mask *dont_mask, char **reason, Timeval *expire);
 static int	ignore_change			(Ignore *, int, void *);
 static int	ignore_list			(Ignore *, int, void *);
 static int	foreach_ignore			(const char *, int, 
@@ -154,15 +149,15 @@ static Ignore *new_ignore (const char *new_nick)
 	upper(item->nick);
 	item->reason = NULL;
 	item->refnum = ++global_ignore_refnum;
-	item->type = 0;
-	item->dont = 0;
-	item->high = 0;
+	mask_unsetall(&item->type);
+	mask_unsetall(&item->dont);
 	item->counter = 0;
 	get_time(&item->creation);
 	item->last_used.tv_sec = 0;
 	item->last_used.tv_usec = 0;
 	item->expiration.tv_sec = 0;
 	item->expiration.tv_usec = 0;
+	item->enabled = 1;
 	add_to_list((List **)&ignored_nicks, (List *)item);
 	return item;
 }
@@ -292,7 +287,9 @@ static int 	remove_ignore (const char *nick)
 		strlcpy(new_nick, nick, IRCD_BUFFER_SIZE);
 	else
 		snprintf(new_nick, IRCD_BUFFER_SIZE, "%s!%s@%s",
-			mnick, user, host);
+				*mnick ? mnick : star,
+				*user ? user : star,
+				*host ? host : star);
 
 	/*
 	 * Look for an exact match first.
@@ -343,33 +340,24 @@ static int 	remove_ignore (const char *nick)
  *  must make a copy of it.  You must not try to write to the string.
  */
 #define HANDLE_TYPE(x,y)						\
-	if ((item->dont & IGNORE_ ## x) == IGNORE_ ## x)		\
+	if (mask_isset(&item->dont, LEVEL_ ## x))			\
 	{								\
 	    if ((y) == 1)						\
 		strlcat_c(buffer, " DONT-" #x, sizeof buffer, &clue);	\
 	    else if ((y) == 2)						\
 		strlcat_c(buffer, " ^" #x, sizeof buffer, &clue);	\
 	}								\
-	else if ((item->type & IGNORE_ ## x) == IGNORE_ ## x)		\
+	else if (mask_isset(&item->type, LEVEL_ ## x))			\
 	{								\
 	    if ((y) == 1)						\
 		strlcat_c(buffer, " " #x, sizeof buffer, &clue);	\
 	    else if ((y) == 2)						\
 		strlcat_c(buffer, " /" #x, sizeof buffer, &clue);	\
 	}								\
-	else if ((item->high & IGNORE_ ## x) == IGNORE_ ## x)		\
-	{								\
-	    if ((y) == 1)						\
-		strlopencat_c(buffer, sizeof buffer, &clue, space, 	\
-				high, #x, high, NULL);			\
-	    else if ((y) == 2)						\
-		strlcat_c(buffer, " +" #x, sizeof buffer, &clue);	\
-	}
 
 static const char *	get_ignore_types (Ignore *item, int output_type)
 {
 static	char 	buffer[BIG_BUFFER_SIZE + 1];
-	char	*high = highlight_char;
 	size_t	clue;
 	char	*retval;
 
@@ -378,19 +366,36 @@ static	char 	buffer[BIG_BUFFER_SIZE + 1];
 	HANDLE_TYPE(ALL, output_type)
 	else
 	{
-		HANDLE_TYPE(MSGS, output_type)
 		HANDLE_TYPE(PUBLIC, output_type)
-		HANDLE_TYPE(WALLS, output_type)
-		HANDLE_TYPE(WALLOPS, output_type)
-		HANDLE_TYPE(INVITES, output_type)
-		HANDLE_TYPE(NOTICES, output_type)
-		HANDLE_TYPE(NOTES, output_type)
-		HANDLE_TYPE(CTCPS, output_type)
-		HANDLE_TYPE(TOPICS, output_type)
-		HANDLE_TYPE(NICKS, output_type)
-		HANDLE_TYPE(JOINS, output_type)
-		HANDLE_TYPE(PARTS, output_type)
-		HANDLE_TYPE(CRAP, output_type)
+		HANDLE_TYPE(MSG, output_type)
+		HANDLE_TYPE(NOTICE, output_type)
+		HANDLE_TYPE(WALL, output_type)
+		HANDLE_TYPE(WALLOP, output_type)
+		HANDLE_TYPE(OPNOTE, output_type)
+		HANDLE_TYPE(SNOTE, output_type)
+		HANDLE_TYPE(ACTION, output_type)
+		HANDLE_TYPE(DCC, output_type)
+		HANDLE_TYPE(CTCP, output_type)
+		HANDLE_TYPE(INVITE, output_type)
+		HANDLE_TYPE(JOIN, output_type)
+		HANDLE_TYPE(NICK, output_type)
+		HANDLE_TYPE(TOPIC, output_type)
+		HANDLE_TYPE(PART, output_type)
+		HANDLE_TYPE(QUIT, output_type)
+		HANDLE_TYPE(KICK, output_type)
+		HANDLE_TYPE(MODE, output_type)
+		HANDLE_TYPE(OTHER, output_type)
+		HANDLE_TYPE(OPERWALL, output_type)
+		HANDLE_TYPE(SYSERR, output_type)
+		HANDLE_TYPE(USER1, output_type)
+		HANDLE_TYPE(USER2, output_type)
+		HANDLE_TYPE(USER4, output_type)
+		HANDLE_TYPE(USER5, output_type)
+		HANDLE_TYPE(USER6, output_type)
+		HANDLE_TYPE(USER7, output_type)
+		HANDLE_TYPE(USER8, output_type)
+		HANDLE_TYPE(USER9, output_type)
+		HANDLE_TYPE(USER10, output_type)
 	}
 
 	retval = buffer;
@@ -409,7 +414,6 @@ static	char 	buffer[BIG_BUFFER_SIZE + 1];
  *		  included, the following variables will be changed:
  *  'do_mask' 	- Suppressive ignores
  *  'dont_mask' - Exceptional ignores
- *  'high_mask' - Highlight ignores
  *  'reason' 	- The reason the ignore was created.  May be NULL.
  *		  Expects an argument.  If argument not provided or argument
  *		  is a hyphen ("-"), the reason is unset.
@@ -425,20 +429,20 @@ static	char 	buffer[BIG_BUFFER_SIZE + 1];
  *	-<TYPE>		Remove <TYPE> from all disposition types
  *	!<TYPE>		Change <TYPE> to an exceptional ignore
  *	^<TYPE>		Same as !<TYPE>
- *	+<TYPE>		Change <TYPE> to a highlight ignore
  *	/<TYPE>		Change <TYPE> to a suppressive ignore
  *	<TYPE>		Same as /<TYPE>
  *
  *  The supported <TYPE>s are defined at the top of this file in the 
  *	"IGNORE THEORY" documentation.
  */
-static int	change_ignore_mask_by_desc (const char *type, int *do_mask, int *dont_mask, int *high_mask, char **reason, Timeval *expire)
+static int	change_ignore_mask_by_desc (const char *type, Mask *do_mask, Mask *dont_mask, char **reason, Timeval *expire)
 {
 	char	*l1, *l2;
 	int	len;
-	int	*mask = NULL, *del1, *del2, *del3;
+	Mask	*mask = NULL, *del1, *del2, *del3;
 	char *	copy;
 	int	bit;
+	int	i;
 
 	copy = LOCAL_COPY(type);
         while ((l1 = new_next_arg(copy, &copy)))
@@ -452,28 +456,21 @@ static int	change_ignore_mask_by_desc (const char *type, int *do_mask, int *dont
 				mask = NULL;
 				del1 = do_mask;
 				del2 = dont_mask;
-				del3 = high_mask;
+				del3 = NULL;
 				break;
 			case '!':
 			case '^':
 				l2++;
 				mask = dont_mask;
 				del1 = do_mask;
-				del2 = high_mask;
-				del3 = NULL;
-				break;
-			case '+':
-				l2++;
-				mask = high_mask;
-				del1 = do_mask;
-				del2 = dont_mask;
+				del2 = NULL;
 				del3 = NULL;
 				break;
 			case '/':
 			default:
 				mask = do_mask;
 				del1 = dont_mask;
-				del2 = high_mask;
+				del2 = NULL;
 				del3 = NULL;
 				break;
 		}
@@ -481,43 +478,7 @@ static int	change_ignore_mask_by_desc (const char *type, int *do_mask, int *dont
 		if (!(len = strlen(l2)))
 			continue;
 
-		if (!my_strnicmp(l2, "NONE", len))
-		{
-			mask = NULL;
-			del1 = do_mask;
-			del2 = dont_mask;
-			del3 = high_mask;
-			bit = IGNORE_ALL;
-		}
-		else if (!my_strnicmp(l2, "ALL", len))
-			bit = IGNORE_ALL;
-		else if (!my_strnicmp(l2, "MSGS", len))
-			bit = IGNORE_MSGS;
-		else if (!my_strnicmp(l2, "PUBLIC", len))
-			bit = IGNORE_PUBLIC;
-		else if (!my_strnicmp(l2, "WALLS", len))
-			bit = IGNORE_WALLS;
-		else if (!my_strnicmp(l2, "WALLOPS", len))
-			bit = IGNORE_WALLOPS;
-		else if (!my_strnicmp(l2, "INVITES", len))
-			bit = IGNORE_INVITES;
-		else if (!my_strnicmp(l2, "NOTICES", len))
-			bit = IGNORE_NOTICES;
-		else if (!my_strnicmp(l2, "NOTES", len))
-			bit = IGNORE_NOTES;
-		else if (!my_strnicmp(l2, "CTCPS", len))
-			bit = IGNORE_CTCPS;
-		else if (!my_strnicmp(l2, "TOPICS", len))
-			bit = IGNORE_TOPICS;
-		else if (!my_strnicmp(l2, "NICKS", len))
-			bit = IGNORE_NICKS;
-		else if (!my_strnicmp(l2, "JOINS", len))
-			bit = IGNORE_JOINS;
-		else if (!my_strnicmp(l2, "PARTS", len))
-			bit = IGNORE_PARTS;
-		else if (!my_strnicmp(l2, "CRAP", len))
-			bit = IGNORE_CRAP;
-		else if (!my_strnicmp(l2, "REASON", len))
+		if (!my_strnicmp(l2, "REASON", len))
 		{
 			char *the_reason;
 
@@ -557,29 +518,48 @@ static int	change_ignore_mask_by_desc (const char *type, int *do_mask, int *dont
 				*expire = time_add(right_now, interval);
 
 				add_timer(0, empty_string, seconds, 1, 
-					  do_expire_ignores, NULL, NULL, -1);
+					  do_expire_ignores, NULL, NULL, 
+					  GENERAL_TIMER, -1, 0);
 			    }
 			}
 
 			continue;
 		}
+
+		/* It's a level of some sort */
+		if (!my_strnicmp(l2, "NONE", len))
+		{
+			mask = NULL;
+			del1 = do_mask;
+			del2 = dont_mask;
+			del3 = NULL;
+			bit = LEVEL_ALL;
+		}
+		else if (!my_strnicmp(l2, "ALL", len))
+			bit = LEVEL_ALL;
 		else
 		{
+		    i = str_to_level(l2);
+		    if (i != -1)
+			    bit = i;
+		    else
+		    {
+			char *levels = get_all_levels();
 			say("You must specify one of the following:");
-			say("\tALL MSGS PUBLIC WALLS WALLOPS INVITES "
-				"NOTICES NOTES CTCPS TOPICS NICKS JOINS "
-				"PARTS CRAP NONE REASON \"<reason>\"");
+			say("\tALL %s NONE REASON \"<reason>\" TIMEOUT <seconds>", levels);
+			new_free(&levels);
 			continue;
+		    }
 		}
 
 		if (mask)
-			*mask |= bit;
+			mask_set(mask, bit);
 		if (del1)
-			*del1 &= ~(bit);
+			mask_unset(del1, bit);
 		if (del2)
-			*del2 &= ~(bit);
+			mask_unset(del2, bit);
 		if (del3)
-			*del3 &= ~(bit);
+			mask_unset(del3, bit);
 	    }
 	}
 
@@ -609,7 +589,7 @@ static int	ignore_change (Ignore *item, int type, void *data)
 
 	changes = (char *)data;
 	change_ignore_mask_by_desc(changes, &item->type, &item->dont, 
-					&item->high, &item->reason, 
+					&item->reason, 
 					&item->expiration);
 
 	/*
@@ -617,9 +597,7 @@ static int	ignore_change (Ignore *item, int type, void *data)
 	 * Garbage collect this ignore if it is clear.
 	 * remove_ignore() does the output for us here.
 	 */
-	if ((item->type & IGNORE_ALL) == IGNORE_NONE && 
-	    (item->high & IGNORE_ALL) == IGNORE_NONE &&
-	    (item->dont & IGNORE_ALL) == IGNORE_NONE)
+	if (mask_isnone(&item->type) && mask_isnone(&item->dont))
 	{
 		remove_ignore(item->nick);
 		return 0;
@@ -741,8 +719,8 @@ static int	foreach_ignore (const char *nicklist, int create, int (*callback) (Ig
 	/*
 	 * Walk over 'nicklist', which separates each target with spaces
 	 * or with commas, ie:
-	 *	/ignore "hop hop2" ALL
-	 *	/ignore hop,hop2 ALL
+	 *	/ignore "nick1 nick2" ALL
+	 *	/ignore nick1,nick2 ALL
 	 */
 	copy = LOCAL_COPY(nicklist);
         while ((arg = new_next_arg(copy, &copy)))
@@ -773,7 +751,9 @@ static int	foreach_ignore (const char *nicklist, int create, int (*callback) (Ig
 			strlcpy(new_nick, nick, sizeof new_nick);
 		    else
 			snprintf(new_nick, sizeof new_nick, "%s!%s@%s",
-					mnick, user, host);
+					*mnick ? mnick : star,
+					*user ? user : star,
+					*host ? host : star);
 
 		    /*
 		     * Create a new ignore item if this one does not exist.
@@ -833,11 +813,10 @@ static int	foreach_ignore (const char *nicklist, int create, int (*callback) (Ig
  *  <level-dispositions> := <disposition> <level>
  *  <disposition> :=   [!|^] 			Set exclusionary ignore 
  *                   | -			Remove level from all types
- *		     | +			Set highlight ignore
  *		     | [/|<empty string>]	Set suppressive ignore
  *  <level> := MSGS | PUBLIC | WALLS | WALLOPS | INVITES |
- *	       NOTICES | NOTES | CTCPS | TOPICS | NICKS | 
- *	       JOINS | PARTS | CRAP | NONE | ALL | 
+ *	       NOTICES | CTCPS | TOPICS | NICKS | 
+ *	       JOINS | PARTS | OTHER | NONE | ALL | 
  *	       REASON "<reason>" | TIMEOUT "<number>"
  *
  * Examples:
@@ -904,7 +883,7 @@ char 	*get_ignores_by_pattern (char *patterns, int covered)
 		{
 			if (covered ? wild_match(tmp->nick, pattern)
 				    : wild_match(pattern, tmp->nick))
-				malloc_strcat_word_c(&retval, space, tmp->nick, &clue);
+				malloc_strcat_word_c(&retval, space, tmp->nick, DWORD_NO, &clue);
 		}
 	}
 
@@ -956,40 +935,48 @@ const char	*get_ignore_types_by_pattern (char *pattern)
 char	*get_ignore_patterns_by_type (char *ctype)
 {
 	Ignore	*tmp;
-	int	do_mask = 0, dont_mask = 0, high_mask = 0;
+	Mask	do_mask, dont_mask;
 	char	*result = NULL;
 	size_t	clue = 0;
 
+	mask_unsetall(&do_mask);
+	mask_unsetall(&dont_mask);
+
 	/*
 	 * Convert the user's input into something we can use.
 	 * If the user doesnt specify anything useful, then we
 	 * just punt right here.
 	 */
-	change_ignore_mask_by_desc(ctype, &do_mask, &dont_mask, &high_mask, NULL, NULL);
-	if (do_mask == 0 && dont_mask == 0 && high_mask == 0)
+	change_ignore_mask_by_desc(ctype, &do_mask, &dont_mask, NULL, NULL);
+	if (mask_isnone(&do_mask) && mask_isnone(&do_mask))
 		return malloc_strdup(empty_string);
 
 	for (tmp = ignored_nicks; tmp; tmp = tmp->next)
 	{
-		/*
-		 * change_ignore_mask_by_desc is supposed to ensure that
-		 * each bit is set only once in 'do_mask', 'dont_mask', 
-		 * and 'high_mask', and we already know this is the case
-		 * for the Ignores.  So we check each of the three ignore
-		 * types, and if this ignore has all of the levels set in
-		 * all of the right places, it's ok.  It could have more 
-		 * levels than what the user asked for, but it can't have 
-		 * levels with different dispositions.
-		 */
-		if ((tmp->dont & dont_mask) != dont_mask)
-			continue;
-		if ((tmp->type & do_mask) != do_mask)
-			continue;
-		if ((tmp->high & high_mask) != high_mask)
-			continue;
+	    int	i;
 
-		/* Add it to the fray */
-		malloc_strcat_word_c(&result, space, tmp->nick, &clue);
+	    /*
+	     * change_ignore_mask_by_desc is supposed to ensure that
+	     * each bit is set only once in 'do_mask', and 'dont_mask', 
+	     * and we already know this is the case
+	     * for the Ignores.  So we check each of the three ignore
+	     * types, and if this ignore has all of the levels set in
+	     * all of the right places, it's ok.  It could have more 
+	     * levels than what the user asked for, but it can't have 
+	     * levels with different dispositions.
+	     */
+	    for (i = 1; BIT_VALID(i); i++)
+	    {
+		if (mask_isset(&dont_mask, i) && !mask_isset(&tmp->dont, i))
+			goto bail;
+		if (mask_isset(&do_mask, i) && !mask_isset(&tmp->type, i))
+			goto bail;
+	    }
+
+	    /* Add it to the fray */
+	    malloc_strcat_word_c(&result, space, tmp->nick, DWORD_NO, &clue);
+bail:
+	    continue;
 	}
 
 	return result;
@@ -1014,7 +1001,6 @@ char	*get_ignore_patterns_by_type (char *ctype)
  *	LEVELS		A parsable summary of what is being ignored
  *	TYPE		An integer representing the suppressive ignores
  *	DONT		An integer representing the exceptional ignores
- *	HIGH		An integer representing the highlight ignores
  *	EXPIRATION	The time the ignore will expire
  *	REASON		The reason why we're ignoring this pattern.
  */ 
@@ -1026,7 +1012,7 @@ char *	ignorectl (char *input)
 	int	len;
 	int	owd;
 
-	GET_STR_ARG(listc, input);
+	GET_FUNC_ARG(listc, input);
 	len = strlen(listc);
 	if (!my_strnicmp(listc, "REFNUM", len)) {
 		if ((i = (Ignore *)find_in_list((List **)&ignored_nicks,
@@ -1040,12 +1026,18 @@ char *	ignorectl (char *input)
 
 		for (i = ignored_nicks; i; i = i->next)
 			malloc_strcat_word_c(&retval, space, 
-						ltoa(i->refnum), &clue);
+						ltoa(i->refnum), DWORD_NO, &clue);
 		RETURN_MSTR(retval);
+	} else if (!my_strnicmp(listc, "SUSPEND", len)) {
+		ignores_are_suspended++;
+	} else if (!my_strnicmp(listc, "UNSUSPEND", len)) {
+		ignores_are_suspended--;
+	} else if (!my_strnicmp(listc, "RESET_SUSPEND", len)) {
+		ignores_are_suspended = 0;
 	} else if (!my_strnicmp(listc, "ADD", len)) {
 		char *	pattern;
 
-		GET_STR_ARG(pattern, input);
+		GET_FUNC_ARG(pattern, input);
 
 		owd = window_display;
 		window_display = 0;
@@ -1056,7 +1048,7 @@ char *	ignorectl (char *input)
 	} else if (!my_strnicmp(listc, "CHANGE", len)) {
 		int	refnum;
 
-		GET_STR_ARG(refstr, input);
+		GET_FUNC_ARG(refstr, input);
 		if (!is_number(refstr))
 			RETURN_EMPTY;
 		refnum = my_atol(refstr);
@@ -1069,7 +1061,13 @@ char *	ignorectl (char *input)
 		window_display = owd;
 		RETURN_INT(i->refnum);
 	} else if (!my_strnicmp(listc, "DELETE", len)) {
-		RETURN_INT(remove_ignore(input));
+		int	retval;
+
+		owd = window_display;
+		window_display = 0;
+		retval = remove_ignore(input);
+		window_display = owd;
+		RETURN_INT(retval);
 	} else if (!my_strnicmp(listc, "PATTERN", len)) {
 		RETURN_MSTR(get_ignores_by_pattern(input, 0));
 	} else if (!my_strnicmp(listc, "RPATTERN", len)) {
@@ -1079,25 +1077,23 @@ char *	ignorectl (char *input)
 	} else if (!my_strnicmp(listc, "GET", len)) {
 		int	refnum;
 
-		GET_STR_ARG(refstr, input);
+		GET_FUNC_ARG(refstr, input);
 		if (!is_number(refstr))
 			RETURN_EMPTY;
 		refnum = my_atol(refstr);
 		if (!(i = get_ignore_by_refnum(refnum)))
 			RETURN_EMPTY;
 
-		GET_STR_ARG(listc, input);
+		GET_FUNC_ARG(listc, input);
 		len = strlen(listc);
 		if (!my_strnicmp(listc, "NICK", len)) {
 			RETURN_STR(i->nick);
 		} else if (!my_strnicmp(listc, "LEVELS", len)) {
 			RETURN_STR(get_ignore_types(i, 2));
 		} else if (!my_strnicmp(listc, "SUPPRESS", len)) {
-			RETURN_INT(i->type);
+			RETURN_INT(i->type.__bits[0]);
 		} else if (!my_strnicmp(listc, "EXCEPT", len)) {
-			RETURN_INT(i->dont);
-		} else if (!my_strnicmp(listc, "HIGHLIGHT", len)) {
-			RETURN_INT(i->high);
+			RETURN_INT(i->dont.__bits[0]);
 		} else if (!my_strnicmp(listc, "EXPIRATION", len)) {
 			char *ptr = NULL;
 			return malloc_sprintf(&ptr, "%ld %ld", 
@@ -1117,41 +1113,50 @@ char *	ignorectl (char *input)
 			return malloc_sprintf(&ptr, "%ld %ld", 
 				(long) i->last_used.tv_sec,
 				(long) i->last_used.tv_usec);
+		} else if (!my_strnicmp(listc, "ENABLED", len)) {
+			RETURN_INT(i->enabled);
 		}
 	} else if (!my_strnicmp(listc, "SET", len)) {
 		int	refnum;
 
-		GET_STR_ARG(refstr, input);
+		GET_FUNC_ARG(refstr, input);
 		if (!is_number(refstr))
 			RETURN_EMPTY;
 		refnum = my_atol(refstr);
 		if (!(i = get_ignore_by_refnum(refnum)))
 			RETURN_EMPTY;
 
-		GET_STR_ARG(listc, input);
+		GET_FUNC_ARG(listc, input);
 		len = strlen(listc);
 		if (!my_strnicmp(listc, "NICK", len)) {
 			malloc_strcpy(&i->nick, input);
 			RETURN_INT(i->refnum);
 		} else if (!my_strnicmp(listc, "LEVELS", len)) {
-			i->type = i->dont = i->high = 0;
+			mask_unsetall(&i->type);
+			mask_unsetall(&i->dont);
 			ignore_change(i, 1, input);
 			RETURN_INT(i->refnum);
 		} else if (!my_strnicmp(listc, "SUPPRESS", len)) {
-			GET_INT_ARG(i->type, input);
+			GET_INT_ARG(i->type.__bits[0], input);
 			RETURN_INT(i->refnum);
 		} else if (!my_strnicmp(listc, "EXCEPT", len)) {
-			GET_INT_ARG(i->dont, input);
-			RETURN_INT(i->refnum);
-		} else if (!my_strnicmp(listc, "HIGHLIGHT", len)) {
-			GET_INT_ARG(i->high, input);
+			GET_INT_ARG(i->dont.__bits[0], input);
 			RETURN_INT(i->refnum);
 		} else if (!my_strnicmp(listc, "EXPIRATION", len)) {
 			Timeval to;
+			Timeval right_now;
+			double seconds;
 
 			GET_INT_ARG(to.tv_sec, input);
 			GET_INT_ARG(to.tv_usec, input);
 			i->expiration = to;
+
+			get_time(&right_now);
+			seconds = time_diff(right_now, to);
+			add_timer(0, empty_string, seconds, 1,
+				do_expire_ignores, NULL, NULL, 
+				GENERAL_TIMER, -1, 0);
+
 			RETURN_INT(i->refnum);
 		} else if (!my_strnicmp(listc, "REASON", len)) {
 			if (is_string_empty(input))
@@ -1176,6 +1181,9 @@ char *	ignorectl (char *input)
 		} else if (!my_strnicmp(listc, "COUNTER", len)) {
 			GET_INT_ARG(i->counter, input);
 			RETURN_INT(i->refnum);
+		} else if (!my_strnicmp(listc, "ENABLED", len)) {
+			GET_INT_ARG(i->enabled, input);
+			RETURN_INT(i->refnum);
 		}
 	}
 
@@ -1195,9 +1203,9 @@ char *	ignorectl (char *input)
  *        ignore instead of the old nick and userhost patterns.
  * (jfn may 1995)
  */
-int	check_ignore (const char *nick, const char *uh, int type)
+int	check_ignore (const char *nick, const char *uh, int mask)
 {
-	return check_ignore_channel(nick, uh, NULL, type);
+	return check_ignore_channel(nick, uh, NULL, mask);
 }
 
 /*
@@ -1205,7 +1213,7 @@ int	check_ignore (const char *nick, const char *uh, int type)
  * i was not inspired with a better name.  This function is simply the
  * old 'check_ignore', only it has an additional check for a channel target.
  */
-int	check_ignore_channel (const char *nick, const char *uh, const char *channel, int type)
+int	check_ignore_channel (const char *nick, const char *uh, const char *channel, int level)
 {
 	char 	nuh[IRCD_BUFFER_SIZE];
 	Ignore	*tmp;
@@ -1218,11 +1226,27 @@ int	check_ignore_channel (const char *nick, const char *uh, const char *channel,
 	if (!ignored_nicks)
 		return NOT_IGNORED;
 
-	snprintf(nuh, IRCD_BUFFER_SIZE - 1, "%s!%s", nick ? nick : star,
+	if (ignores_are_suspended)
+		return NOT_IGNORED;
+
+	/* 
+	 * If the userhost is the empty string (but not NULL)
+	 * then because of how ParseArgs() works, we know that
+	 * 'nick' contains a (remote) server name.  Yes, you 
+	 * can ignore (remote) servers.
+	 */
+	if (nick && strchr(nick, '.') && uh && !*uh)
+	    snprintf(nuh, IRCD_BUFFER_SIZE - 1, "%s", nick);
+	else
+	    snprintf(nuh, IRCD_BUFFER_SIZE - 1, "%s!%s", 
+						nick ? nick : star,
 						uh ? uh : star);
 
 	for (tmp = ignored_nicks; tmp; tmp = tmp->next)
 	{
+		if (!tmp->enabled)
+			continue;
+
 		/*
 		 * Always check for exact matches first...
 		 */
@@ -1264,24 +1288,18 @@ int	check_ignore_channel (const char *nick, const char *uh, const char *channel,
 	if (i_match)
 	{
 		tmp = i_match;
-		if (tmp->dont & type)
+		if (mask_isset(&tmp->dont, level))
 		{
 			tmp->counter++;
 			get_time(&tmp->last_used);
 			return NOT_IGNORED;
 		}
-		if (tmp->type & type)
+		if (mask_isset(&tmp->type, level))
 		{
 			tmp->counter++;
 			get_time(&tmp->last_used);
 			return IGNORED;
 		}
-		if (tmp->high & type)
-		{
-			tmp->counter++;
-			get_time(&tmp->last_used);
-			return HIGHLIGHTED;
-		}
 	}
 
 	/*
@@ -1291,24 +1309,18 @@ int	check_ignore_channel (const char *nick, const char *uh, const char *channel,
 	else if (c_match)
 	{
 		tmp = c_match;
-		if (tmp->dont & type)
+		if (mask_isset(&tmp->dont, level))
 		{
 			tmp->counter++;
 			get_time(&tmp->last_used);
 			return NOT_IGNORED;
 		}
-		if (tmp->type & type)
+		if (mask_isset(&tmp->type, level))
 		{
 			tmp->counter++;
 			get_time(&tmp->last_used);
 			return IGNORED;
 		}
-		if (tmp->high & type)
-		{
-			tmp->counter++;
-			get_time(&tmp->last_used);
-			return HIGHLIGHTED;
-		}
 	}
 
 	/*
